/****************************************************************************
* Program/file: Main.c
*
* Copyright (C) by RTX A/S, Denmark.
* These computer program listings and specifications, are the property of
* RTX A/S, Denmark and shall not be reproduced or copied or used in
* whole or in part without written permission from RTX A/S, Denmark.
*
* DESCRIPTION: Co-Located Application (COLA).
*
****************************************************************************/

/****************************************************************************
*                                  PVCS info
*****************************************************************************

$Author:   mmj  $
$Date:   11 Nov 2013 10:16:30  $
$Revision:   1.16  $
$Modtime:   11 Nov 2013 10:16:06  $
$Archive:   J:/sw/Projects/Amelie/COLApps/Apps/WebServer/vcs/Main.c_v  $

*/

/****************************************************************************
*                               Include files
****************************************************************************/

#include <Core/RtxCore.h>
#include <Ros/RosCfg.h>
#include <PortDef.h>
#include <Api/Api.h>
#include <Cola/Cola.h>
#include <Protothreads/Protothreads.h>
#include <SwClock/SwClock.h>
#include <NetUtils/NetUtils.h>

#include <PtApps/AppCommon.h>
#include <PtApps/AppLed.h>
#include <PtApps/AppSocket.h>
#include <PtApps/AppWifi.h>
#include <PtApps/AppSntp.h>
#include <PtApps/AppWebconfig.h>
#include <Drivers/DrvButtons.h>  
#include <Drivers/DrvIntTemp.h>
#include <Drivers/DrvApds990x.h>
#include <Drivers/DrvHIH613x.h>
#include <Drivers/DrvLps331ap.h>
#include <Drivers/DrvI2cIntf.h>
#include <Drivers/DrvLsm303dlhc.h>
#include <Drivers/DrvNtcTemp.h>
#include <Drivers/DrvAdc.h>
#include <Drivers/DrvMvs0409.h>
#include <Drivers/DrvSpiSlave.h>
#include <Drivers/DrvSpi.h>

#include <3Party/HttpParser/http_parser.h>
  
/****************************************************************************
*                              Macro definitions
****************************************************************************/
#define STATIC_IP   "192.168.1.135"
#define SUBNET_MASK "255.255.255.0"
#define GATEWAY     "192.168.1.1"
#define HTTP_PORT 80
#define RX_IDLE_TIMEOUT  (30*RS_T1SEC)
#define DATE_STR_LEN 30

// Data for configure softAP
#define SOFT_AP_ESSID           "SCK_AP_" //partial essid
#define SOFT_AP_SECURITY_TYPE   AWST_NONE
#define SOFT_AP_KEY           "rtx41xxpass"
#define SOFT_AP_COUNTRY_CODE    "FF" //ES
#define SOFT_AP_CHANNEL         2437//Channel 6 center frequency
#define SOFT_AP_INACT           10 // minutes
#define SOFT_AP_BEACON          100 // ms
#define SOFT_AP_IP              "192.168.1.135"
//#define SOFT_AP_SUBNET_MASK "255.255.255.0"
//#define SOFT_AP_GATEWAY     "192.168.1.1"

#define SOFT_AP_DHCP_POOL_START "192.168.1.200"
#define SOFT_AP_DHCP_POOL_END   "192.168.1.255"
#define SOFT_AP_SUBNET          "255.255.255.1"
#define SOFT_AP_LEASE_TIME      86400
/****************************************************************************
*                     Enumerations/Type definitions/Structs
****************************************************************************/
typedef struct
{
  struct pt ChildPt;
  PtEntryType* PtEntryPtr; 
  rschar DateStr[DATE_STR_LEN];
  rsint16 Temperature;
  rsint16 Humidity;
  rsint32 Pressure;
  rsint16 Lux;
  rsint32 SSID;
  rsbool MultiSensor;
  rsbool HttpServerStarted;
  rsuint32 HttpInstance;
} AppDataType;

/****************************************************************************
*                            Global variables/const
****************************************************************************/

/****************************************************************************
*                            Local variables/const
****************************************************************************/
static const char* const Day[] = { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" };
static const char* const Month[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};

static RsListEntryType PtList;
static AppDataType AppData;

// Timers
static const RosTimerConfigType PacketDelayTimer =
  ROSTIMER(COLA_TASK, APP_PACKET_DELAY_TIMEOUT,
  APP_PACKET_DELAY_TIMER);

static const RosTimerConfigType DnsRspTimer =
  ROSTIMER(COLA_TASK, APP_DNS_RSP_TIMEOUT,
  APP_DNS_RSP_TIMER);

// TCP flags
static char TCP_is_connected; // True when the TCP connection has been stablished
static char TCP_received; // True when data has been received at the TCP connection

static int socketHandle; // The socket ID of the TCP connection
static rsuint8 *TCP_receive_buffer_ptr; // TCP receive buffer
static int TCP_Rx_bufferLength; // Number of bytes received at the TCP buffer

// Energy control
static rsuint8 is_suspended;
/****************************************************************************
*                          Local Function prototypes
****************************************************************************/

/****************************************************************************
*                                Implementation
***************************************************************************/
/*** Genera la capçalera d’HTTP ***/
static rsuint32 AddHeader(rsuint8 *BufferPtr, rsuint32 BufferLength, rsuint8* DataPtr, rsuint32 Instance)
{
  rsuint8 *p = BufferPtr;
  rsuint32 l = 0;

  l += HttpAddHeader(p+l, BufferLength-l, "Content-Type", "text/html; charset=utf-8");
  l += HttpAddHeader(p+l, BufferLength-l, "Server", "RTX41xx demo WEB server");
  l += HttpAddHeader(p+l, BufferLength-l, "Date", AppData.DateStr);

  return l;
}

/*** Genera la web HTML, aquí és on hem de ficar el nostre codi ***/
static rsuint32 GenerateMainPage(rsuint8 *BufferPtr, rsuint32 BufferLength, rsuint32 Offset, rsuint8 *DataPtr, rsuint32 Instance)
{
  rsuint16 n = 0;
  rsuint16 l = BufferLength;
  rschar *p = (rschar*)BufferPtr;
  const char* const start = 
    /***CODI HTML***/
    "<!DOCTYPE html>"
    "<html>"
      "<head>"
        "<meta charset=\"utf-8\" />"
        "<title>WEB SERVER RTX4100</title>"
      "<style>" //Codi CSS
      "body{"
        "width: 500px;"
        "margin: auto;"
        "background-color: #E2ECEE;"
        "color: FF0000;"
      "}"

      ".Contenedor {"
      "background-color: #FFFFFF;"
      "color: #000000;"
      "border: 6px groove #0E93AD;"
      "font-family: arial;"
      "text-align: justify;"
      "}"

      /*//Codi CSS per les pestanyes
      "#pestanas {"
       "float: top;"
       "font-size: 3ex;"
       "font-weight: bold;"
      "}"

      "#pestanas ul{"
        "margin-left: -40px;"   
      "}"

      "#pestanas li{"
      "list-style-type: none;"
      "float: left;"
       "text-align: center;"
       "margin: 0px 2px -2px -0px;"
       "background: darkgrey;"
       "border-top-left-radius: 5px;"
       "border-top-right-radius: 5px;"
       "border: 2px solid bisque;"
       "border-bottom: dimgray;"
       "padding: 0px 20px 0px 20px;"
      "}"

      "#pestanas a:link{"
        "text-decoration: none;"
        "color: bisque;"
       "}"

      "#contenidopestanas{"
        "clear: both;"
        "background: dimgray;"
        "padding: 20px 0px 20px 20px;"
        "border-radius: 5px;"
        "border-top-left-radius: 0px;"
        "border: 2px solid bisque;"
        "width: 1025px;"
      "}" */
      "</style>"
      "</head>"
      
      "<body>"
      "<h1>RTX4100 WEB Server</h1>"
      "<form class='Contenedor'>"
          "<h2>Configuration menu</h2>"
            "<nav>"
              "<ul>"
                "<li><a href=\"setup\">Set IP config</a></li>" //Falta afegir noves pags.
              "</ul>"
            "</nav>" 
            
            "<form class='Contenedor'>" 
            "<h3>System information</h3>"
              "<div><td><b>Date:</b></td><td>%s</td></tr></div>"
              "<div><td><b>Temperature:</b></td><td>%d.%d C</td></tr></div>"
              "<div><td><b>Humidity</b></td><td>%i.%i %s</td></tr></div>";

  const char* const multiSensor = 
          //"<tr><td><b>Pressure:</b></td><td>%i.%i hPa</td></tr>"
          "<tr><td><b>Light:</b></td><td>%i LUX</td></tr>";
          
  const char* const end = 
        "<div><td><b>SSID:</b></td><td>%d</td></tr></div>"
      //"<div><td><b>IP:</b></td><td>%d</td></tr></div>"
        "<br><div><input type='submit' value='Refresh'></br></div>"
      /*//Formularis, no s'ha provat
      "<form action=\"setup\" method=\"POST\">"
        "<input type=\"hidden\" name="subject" value=\"Formulari\">"
        "<table>"
        "<tr><td>IP</td><td><input type="text" name="nombre"></td></tr>"
         "</td></tr>"  
        "<tr>"
        "<td colspan="2" align="center">"
           "<input type='submit' name='Enviar' value='Enviar'>"
            "<input type="reset"></td>"
          "</tr>"
         "</table>"*/
        "</form>"
      "</body>"
    "</html>";
/***FI CODI HTML***/

  n = snprintf(p, l, start, AppData.DateStr, AppData.Temperature/10, AppData.Temperature%10,AppData.Humidity%10, AppData.SSID);
  if (AppData.MultiSensor)
  {
    n += snprintf(p ? p+n : NULL, l, multiSensor, 
                  (int)AppData.Humidity/10, (int)AppData.Humidity%100, "\"%\"RH",
                  (int)AppData.Pressure/100, (int)AppData.Pressure%100, 
                  (int)AppData.Lux);
  }
  n += snprintf(p ? p+n : NULL, l, end);
  return n;
}

/*** Protothread: Traurem el valor de les mesures ***/
static PT_THREAD(PtDoMainPage(struct pt *Pt, const RosMailType *Mail))
{
  static struct pt SensorUpdatePt;

  PT_BEGIN(Pt);

  // 1. Get time //La fecha se tiene que configurar el dia que se encienda la placa para ponerla a la hora .http://playground.arduino.cc/Code/DateTime
  PT_WAIT_UNTIL(Pt, IS_RECEIVED(API_GET_TIME_CFM));
  if (((ApiGetTimeCfmType *)Mail)->Status == RSS_SUCCESS)
  {
    SwcTimeType absTime;
    SwcRelative2AbsoluteTime(&absTime, ((ApiGetTimeCfmType *)Mail)->Time);
    snprintf(AppData.DateStr, DATE_STR_LEN, "%s, %d %s %d %02d:%02d:%02d GMT", 
             Day[absTime.WeekDay], absTime.Day, Month[absTime.Month-1], absTime.Year + 1900, absTime.Hour, absTime.Minute, absTime.Second);
  }

  // 2. Read all sensors and update the sensor data stored in AppExosite
  if(AppData.MultiSensor)
  {
    rsint16 data; 
    rsuint16 udata;
    PT_SPAWN(Pt, &SensorUpdatePt, PtDrvAdc_Measure(&SensorUpdatePt, Mail, &DrvNtcAdcConfig, &udata));
    AppData.Temperature = DrvNtcTempConvertValue(ntcConvTable1, sizeOfNtcConvTable1, udata);
    PT_SPAWN(Pt, &SensorUpdatePt, PtDrvApds990xGetLux(&SensorUpdatePt, Mail, &AppData.Lux));
    PT_SPAWN(Pt, &SensorUpdatePt, PtDrvHIH613xMeasure(&SensorUpdatePt, Mail, &AppData.Humidity));
    PT_SPAWN(Pt, &SensorUpdatePt, PtDrvLps331apMeasure(&SensorUpdatePt, Mail, &AppData.Pressure, &data));
  }
  else
  {
    // Measure internal temperature inside the EFM32 chip.
    PT_SPAWN(Pt, &SensorUpdatePt, PtDrvIntTempMeasure(&SensorUpdatePt, Mail, &AppData.Temperature));
  }
  
  // 3. Send/generate the WEB page
  SendApiHttpServerSendResponseReq(COLA_TASK, AppData.HttpInstance, 200, "OK", 
                                   GenerateMainPage(NULL, 0, 0, NULL, 0), // Calc the size of the main page
                                   NULL,                               // Static body not used  
                                   AddHeader,                          // Call back used to add HTTP headers 
                                   GenerateMainPage);                  // Callback used to generate the main page   
  PT_WAIT_UNTIL(Pt, IS_RECEIVED(API_HTTP_SERVER_SEND_RESPONSE_CFM) && 
                    ((ApiHttpServerSendResponseCfmType*)Mail)->Instance == AppData.HttpInstance);

  // 4. Clear PtEntryPtr to signal that we are ready to handle a new request
  AppData.PtEntryPtr = NULL;

  PT_END(Pt);
}

/*** Enviament HTTP (RSS) ***/
// This function is called when HTTP GET reguest with path = "/" is received
static RsStatusType OnMainPage(AhHttpMethodIdType HttpMethod, rschar *PathPtr, rschar *QueryPtr, rsuint32 Instance)
{
  if (HttpMethod != AHM_GET)
  {
    return RSS_NOT_SUPPORTED;
  }

  // Start a protothread that will read the sensors and generate the HTML page
  if (AppData.PtEntryPtr == NULL)
  {
    // Start a protothread that will wait for get time cfm, do sensor reading 
    // and send HTTP response message with the WEB page generated. 
    AppData.PtEntryPtr = PtStart(&PtList, PtDoMainPage, NULL, NULL);

    // Request current time (CFM is handled by PtDoMainPage())
    SendApiGetTimeReq(COLA_TASK);
  }
  return RSS_SUCCESS;
}

/***CODI softAP***/ /*** Protothread del SoftAP ***/
static PT_THREAD(softAP(struct pt *Pt, const RosMailType *Mail)) {
    static struct pt childPt; //Protothread fill
    PT_BEGIN(Pt);
    
    PT_SPAWN(Pt, &childPt, PtAppWifiReset(&childPt, Mail));
    //PT_SPAWN(Pt, &childPt, PtAppPrologueNoConnect(&childPt, Mail));
    //PT_SPAWN(Pt, &childPt, PtAppWifiPowerOn(&childPt, Mail));
    
    //SSID = MASK + MAC //Creació del SSID
    const ApiWifiMacAddrType *pMacAddr = AppWifiGetMacAddr();
    
    rsuint8 essid[32];
    snprintf((char*)essid, sizeof(essid), "%s%02X%02X%02X", 
            SOFT_AP_ESSID, (*pMacAddr)[3], (*pMacAddr)[4], (*pMacAddr)[5]);
    AppWifiApSetSoftApInfo((rsuint8*)essid, SOFT_AP_SECURITY_TYPE, 
            FALSE, 0, NULL, SOFT_AP_CHANNEL, SOFT_AP_INACT, 
            (rsuint8*)SOFT_AP_COUNTRY_CODE, SOFT_AP_BEACON);
    
   AppData.SSID=essid;

    //Start SoftAP
    PT_SPAWN(Pt, &childPt, PtAppWifiStartSoftAp(&childPt, Mail));

    //static IP
    ApiIpV4AddressType address, subnetMask, gateway;
    inet_aton(STATIC_IP, &address);
    inet_aton(SUBNET_MASK, &subnetMask);
    inet_aton(GATEWAY, &gateway);
    AppWifiIpv4Config(TRUE, address, subnetMask, gateway, 0);

    //SendApiWifiApSetDhcpPoolReq(COLA_TASK, SOFT_AP_DHCP_POOL_START, SOFT_AP_DHCP_POOL_END, SOFT_AP_LEASE_TIME);

    //wait a client
    PT_WAIT_UNTIL(Pt, IS_RECEIVED(API_WIFI_CONNECT_IND));
    
    PT_END(Pt);
}
/***FI CODI softAP***/

/*** Protothread: Crida al SoftAP ***/
static PT_THREAD(PtMain(struct pt *Pt, const RosMailType* Mail))
{
  //Protothread fill
  static struct pt childPt;
  PT_BEGIN(Pt);

  // Set power save parameters  
  AppWifiSetPowerSaveProfile(POWER_SAVE_HIGH_IDLE); // 200ms idle interval
  AppWifiSetListenInterval(100);                    // 100ms 

  //Iniciar el protohothread del softAP com fill
  PT_SPAWN(Pt, &childPt, softAP(&childPt, Mail));

  // Start HTTP server
  SendApiHttpServerInitReq(COLA_TASK, 80, NULL);
  PT_WAIT_UNTIL(Pt, IS_RECEIVED(API_HTTP_SERVER_INIT_CFM) && ((ApiHttpServerInitCfmType*)Mail)->Port == 80);
  if (((ApiHttpServerInitCfmType*)Mail)->Status == RSS_SUCCESS)
  {
    // Server started
    AppData.HttpServerStarted = TRUE;
    AppData.HttpInstance = ((ApiHttpServerInitCfmType*)Mail)->Instance;

    // Add main page
    SendApiHttpServerAddResourceReq(COLA_TASK, AppData.HttpInstance, "/", OnMainPage);
  }
  else
  {
    // FIX ME: Failed to start HTTP server
  }

  // Request current time using SNTP
  AppSntpStartTimeSync(&PtList, NULL);
  PT_WAIT_UNTIL(Pt, IS_RECEIVED(APP_EVENT_SNTP_SYNC_DONE) || IS_RECEIVED(APP_EVENT_SNTP_SYNC_FAILED));

  while (1)
  {
    // Yield to allow other tasks to run
    PT_YIELD(Pt);

    if (IS_RECEIVED(KEY_MESSAGE))
    {
      if (Mail->P1.P1 == KEY_WPS)
      {
        // Stop the HTTP server
        if (AppData.HttpServerStarted)
        {
          SendApiHttpTerminateReq(COLA_TASK, 80);
          PT_WAIT_UNTIL(Pt, IS_RECEIVED(API_HTTP_TERMINATE_CFM) && ((ApiHttpTerminateCfmType*)Mail)->Instance == AppData.HttpInstance);
          AppData.HttpServerStarted = FALSE;
        }

        // Do WPS!
        AppLedSetLedState(LED_STATE_WPS_ONGOING);
        PT_SPAWN(Pt, &AppData.ChildPt, PtAppWifiDoWps(&AppData.ChildPt, Mail));
        AppLedSetLedState(LED_STATE_IDLE);

        // WPS done -> restart to reconnect
        PT_RESTART(Pt);
      }
    }
  }

  PT_END(Pt);
}

/*** SPI ***/
    static PT_THREAD(Ptspi(struct pt *Pt, const RosMailType *Mail)) {
    static struct pt childPt;
    PT_BEGIN(Pt);

        const rsuint32 baud_rate = 32000000;
        
        printf("Arrancando PtDrvSpiSlaveInit\n");
        PT_SPAWN(Pt, &childPt, PtDrvSpiSlaveInit(&childPt, Mail, baud_rate)); 
        
        printf("Arrancando DrvSpiSlaveInit\n");
        DrvSpiSlaveInit(baud_rate);
  
        while (1) {
            PT_WAIT_UNTIL(Pt, IS_RECEIVED(SPI_SLAVE_RX_DATA));
            RosTimerStart(APP_PACKET_DELAY_TIMER, (3000 * RS_T1MS), &PacketDelayTimer);PT_YIELD_UNTIL(Pt, IS_RECEIVED(APP_PACKET_DELAY_TIMEOUT));  
            
            printf("prueba de envios..\n");

            rsuint8 recibidospi[DrvSpiSlaveRxGetSize()]; 
            DrvSpiSlaveRx(&recibidospi[0], sizeof(recibidospi));
            AppData.Temperature = recibidospi[0];
            AppData.Humidity = recibidospi[1];
            static rsuint8 enviadospi[5];

            enviadospi[0]=AppData.Temperature;
            enviadospi[1]=AppData.Humidity;
            
            enviadospi[0] = recibidospi[0];
            enviadospi[1] = recibidospi[1];
          //  enviadospi[2] = recibidospi[2];
         
            DrvSpiSlaveRxFlush();
            
            DrvSpiSlaveTxStart(&enviadospi[0], 3);
            printf("ENVIADO\n");
            
        }

    PT_END(Pt);
}
/*** FI SPI ***/

void ColaTask(const RosMailType* Mail)
{
  // Pre-dispatch mail handling
  switch (Mail->Primitive)
  {
    case INITTASK:
      // Init the Buttons driver
      DrvButtonsInit();

      // Init sensor drivers
      DrvIntTemp_Init();
      if (DrvLps331ap_Init())
      {
        AppData.MultiSensor = TRUE;
        DrvApds990x_Init();
        DrvHIH613x_Init();
        DrvAdc_Init(&DrvNtcAdcConfig);
      }

      // Init the Protothreads lib
      PtInit(&PtList);

      // Init the LED application
      AppLedInit(&PtList);

      // Init the WiFi management application
      AppWifiInit(&PtList);

      // Start the Main protothread
      PtStart(&PtList, PtMain, NULL, NULL);
      break;

    case TERMINATETASK:
      RosTaskTerminated(ColaIf->ColaTaskId);
      break;

    case API_GPIO_INTERRUPT_IND:
      // Dispatch API_GPIO_INTERRUPT_IND to the button driver and return
      DrvButtonsOnMail(Mail);
      return;
  }

  // Dispatch mail to all protothreads started
  PtDispatchMail(&PtList, Mail);
}

// End of file.